#!/usr/bin/env node
"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault")["default"];

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _path = _interopRequireDefault(require("path"));

var _fs = require("fs");

var _prepareUrls = _interopRequireDefault(require("local-ip-url/prepareUrls"));

var _detectPort = _interopRequireDefault(require("detect-port"));

var _safe = _interopRequireDefault(require("colors-cli/safe"));

var _express = _interopRequireDefault(require("express"));

var _minimist = _interopRequireDefault(require("minimist"));

var _ = _interopRequireDefault(require("../"));

var CWD = process.cwd();

var PKG_PATH = _path["default"].resolve(CWD, './package.json');

var DEFAULT_MOCKER_CONFIG_PATH = _path["default"].resolve(CWD, './mocker.config.json');

var DEFAULT_MOCK_PATH = ['./mock'];
var DEFAULT_CONFIG = {
  host: '0.0.0.0',
  port: 3721
};
(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
  var argvs, _require, version, paths, entryFiles, mockConfigPath, mockerConfig, pkgConf, DEFAULT_PORT, DEFAULT_HOST, app;

  return _regenerator["default"].wrap(function _callee$(_context) {
    while (1) {
      switch (_context.prev = _context.next) {
        case 0:
          argvs = (0, _minimist["default"])(process.argv.slice(2));

          if (!(argvs.h || argvs.help)) {
            _context.next = 14;
            break;
          }

          console.log('\n  Usage: mocker <path> [--config] [--help|h]');
          console.log('\n  Displays help information.');
          console.log('\n  Options:');
          console.log('    --config <path>', 'Simple configuration');
          console.log('\n  Example:');
          console.log('    mocker mock/index.js');
          console.log('    mocker mock/index.js --port 7788');
          console.log('    mocker mock/index.js --host 0.0.0.0');
          console.log('    mocker mock/m1.js test/m2.js');
          console.log('    mocker mock/m1.js --config mocker.config.json');
          console.log('\n');
          return _context.abrupt("return");

        case 14:
          // Fix type errors
          _require = require('../../package.json'), version = _require.version;

          if (!(argvs.v || argvs.version)) {
            _context.next = 18;
            break;
          }

          console.log(version);
          return _context.abrupt("return");

        case 18:
          paths = argvs['_'];

          if (!(paths.length === 0)) {
            _context.next = 23;
            break;
          }

          console.log(_safe["default"].red('Error: Need to pass parameters!'));
          console.log("E.g: ".concat(_safe["default"].yellow('mocker <File path>'), "\n"));
          return _context.abrupt("return");

        case 23:
          entryFiles = paths || DEFAULT_MOCK_PATH;
          mockConfigPath = argvs.config || DEFAULT_MOCKER_CONFIG_PATH;
          mockerConfig = DEFAULT_CONFIG;

          if (argvs.config) {
            mockConfigPath = argvs.config;
          }

          if ((0, _fs.existsSync)(_path["default"].resolve(mockConfigPath))) {
            _context.next = 34;
            break;
          }

          mockerConfig.host = process.env.HOST ? process.env.HOST : mockerConfig.host;
          _context.next = 31;
          return (0, _detectPort["default"])(mockerConfig.port);

        case 31:
          mockerConfig.port = _context.sent;
          _context.next = 35;
          break;

        case 34:
          mockerConfig = require(_path["default"].resolve(mockConfigPath));

        case 35:
          /**
           * Support setting configuration on package.json
           * https://github.com/jaywcjlove/mocker-api/issues/144
           */
          if ((0, _fs.existsSync)(PKG_PATH)) {
            pkgConf = require(PKG_PATH);

            if (pkgConf.mocker) {
              Object.assign(mockerConfig, pkgConf.mocker);
            }
          }

          if (argvs.host) {
            mockerConfig.host = argvs.host;
          }

          if (argvs.port) {
            mockerConfig.port = argvs.port;
          }

          DEFAULT_PORT = mockerConfig.port;
          DEFAULT_HOST = mockerConfig.host;
          app = (0, _express["default"])();
          app.all('/*', function (req, res, next) {
            console.log("".concat(_safe["default"].green(req.method), " - ").concat(req.url));
            res.header('Access-Control-Allow-Origin', '*');
            res.header('Access-Control-Allow-Headers', 'Content-Type,Content-Length,Authorization,Accept,X-Requested-With');
            res.header('Access-Control-Allow-Methods', 'PUT,POST,GET,DELETE,OPTIONS');
            next();
          });
          delete mockerConfig.port;
          delete mockerConfig.host;
          (0, _["default"])(app, entryFiles, (0, _objectSpread2["default"])({}, mockerConfig));
          app.listen(DEFAULT_PORT, function () {
            var localIpUrl = (0, _prepareUrls["default"])({
              protocol: 'http',
              host: DEFAULT_HOST,
              port: DEFAULT_PORT
            });
            console.log("> Server Listening at Local: ".concat(_safe["default"].green(localIpUrl.localUrl)));
            console.log(">           On Your Network: ".concat(_safe["default"].green(localIpUrl.lanUrl), "\n"));
          });
          /**
           * Event listener for HTTP server "error" event.
           */

          app.on('error', function (error) {
            if (error.syscall !== 'listen') {
              throw error;
            }

            var bind = typeof DEFAULT_PORT === 'string' ? "Pipe ".concat(DEFAULT_PORT) : "Port ".concat(DEFAULT_PORT); // handle specific listen errors with friendly messages

            switch (error.code) {
              case 'EACCES':
                console.error("".concat(bind, " requires elevated privileges")); // eslint-disable-line

                process.exit(1);
                break;

              case 'EADDRINUSE':
                console.error("".concat(bind, " is already in use")); // eslint-disable-line

                process.exit(1);
                break;

              default:
                throw error;
            }
          });

        case 47:
        case "end":
          return _context.stop();
      }
    }
  }, _callee);
}))();
//# sourceMappingURL=mocker.js.map