"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard")["default"];

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault")["default"];

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = mockerApi;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _path = _interopRequireDefault(require("path"));

var toRegexp = _interopRequireWildcard(require("path-to-regexp"));

var _clearModule = _interopRequireDefault(require("clear-module"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _safe = _interopRequireDefault(require("colors-cli/safe"));

var _proxyHandle = require("./proxyHandle");

var _mockerHandle = require("./mockerHandle");

var pathToRegexp = toRegexp.pathToRegexp;
var mocker = {};

function mockerApi(app, watchFile) {
  var conf = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var watchFiles = (Array.isArray(watchFile) ? watchFile : typeof watchFile === 'string' ? [watchFile] : []).map(function (str) {
    return _path["default"].resolve(str);
  });

  if (watchFiles.some(function (file) {
    return !file;
  })) {
    throw new Error('Mocker file does not exist!.');
  }
  /**
   * Mybe watch file or pass parameters
   * https://github.com/jaywcjlove/mocker-api/issues/116
   */


  var isWatchFilePath = Array.isArray(watchFile) && watchFile.every(function (val) {
    return typeof val === 'string';
  }) || typeof watchFile === 'string';
  mocker = isWatchFilePath ? getConfig() : watchFile;

  if (!mocker) {
    return function (req, res, next) {
      next();
    };
  }

  var options = (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, conf), mocker._proxy || {});
  var defaultOptions = {
    changeHost: true,
    pathRewrite: {},
    proxy: {},
    // proxy: proxyConf: {},
    httpProxy: {},
    // httpProxy: httpProxyConf: {},
    bodyParserConf: {},
    bodyParserJSON: {},
    bodyParserText: {},
    bodyParserRaw: {},
    bodyParserUrlencoded: {},
    watchOptions: {},
    header: {},
    priority: 'proxy',
    withFullUrlPath: false
  };
  options = (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, defaultOptions), options); // changeHost = true,
  // pathRewrite = {},
  // proxy: proxyConf = {},
  // httpProxy: httpProxyConf = {},
  // bodyParserConf= {},
  // bodyParserJSON = {},
  // bodyParserText = {},
  // bodyParserRaw = {},
  // bodyParserUrlencoded = {},
  // watchOptions = {},
  // header = {}

  if (isWatchFilePath) {
    // 监听配置入口文件所在的目录，一般为认为在配置文件/mock 目录下的所有文件
    // 加上require.resolve，保证 `./mock/`能够找到`./mock/index.js`，要不然就要监控到上一级目录了
    var watcher = _chokidar["default"].watch(watchFiles.map(function (watchFile) {
      return _path["default"].dirname(require.resolve(watchFile));
    }), options.watchOptions);

    watcher.on('all', function (event, path) {
      if (event === 'change' || event === 'add') {
        try {
          // 当监听的可能是多个配置文件时，需要清理掉更新文件以及入口文件的缓存，重新获取
          cleanCache(path);
          watchFiles.forEach(function (file) {
            return cleanCache(file);
          });
          mocker = getConfig();

          if (mocker._proxy) {
            options = (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, options), mocker._proxy);
          }

          console.log("".concat(_safe["default"].green_b.black(' Done: '), " Hot Mocker ").concat(_safe["default"].green(path.replace(process.cwd(), '')), " file replacement success!"));
        } catch (ex) {
          console.error("".concat(_safe["default"].red_b.black(' Failed: '), " Hot Mocker ").concat(_safe["default"].red(path.replace(process.cwd(), '')), " file replacement failed!!"));
        }
      }
    });
  } // 监听文件修改重新加载代码
  // 配置热更新


  app.all('/*', function (req, res, next) {
    var getExecUrlPath = function getExecUrlPath(req) {
      return options.withFullUrlPath ? req.url : req.path;
    };
    /**
     * Get Proxy key
     */


    var proxyKey = Object.keys(options.proxy).find(function (kname) {
      return !!pathToRegexp(kname.replace(new RegExp('^' + req.method + ' '), '')).exec(getExecUrlPath(req));
    });
    /**
     * Get Mocker key
     * => `GET /api/:owner/:repo/raw/:ref`
     * => `GET /api/:owner/:repo/raw/:ref/(.*)`
     */

    var mockerKey = Object.keys(mocker).find(function (kname) {
      return !!pathToRegexp(kname.replace(new RegExp('^' + req.method + ' '), '')).exec(getExecUrlPath(req));
    });
    /**
     * Access Control Allow options.
     * https://github.com/jaywcjlove/mocker-api/issues/61
     */

    var accessOptions = (0, _objectSpread2["default"])({
      'Access-Control-Allow-Origin': req.get('Origin') || '*',
      'Access-Control-Allow-Methods': 'POST, GET, OPTIONS, PUT, DELETE',
      'Access-Control-Allow-Headers': 'Content-Type, X-Requested-With,' + (req.header('access-control-request-headers') || ''),
      'Access-Control-Allow-Credentials': 'true'
    }, options.header);
    Object.keys(accessOptions).forEach(function (keyName) {
      res.setHeader(keyName, accessOptions[keyName]);
    }); // fix issue 34 https://github.com/jaywcjlove/mocker-api/issues/34
    // In some cross-origin http request, the browser will send the preflighted options request before sending the request methods written in the code.

    if (!mockerKey && req.method.toLocaleUpperCase() === 'OPTIONS' && Object.keys(mocker).find(function (kname) {
      return !!pathToRegexp(kname.replace(new RegExp('^(PUT|POST|GET|DELETE) '), '')).exec(getExecUrlPath(req));
    })) {
      return res.sendStatus(200);
    }
    /**
     * priority 'proxy' or 'mocker' [#151](https://github.com/jaywcjlove/mocker-api/issues/151)
     */


    if (options.priority === 'mocker') {
      if (mocker[mockerKey]) {
        return (0, _mockerHandle.mockerHandle)({
          req: req,
          res: res,
          next: next,
          mocker: mocker,
          options: options,
          mockerKey: mockerKey
        });
      } else if (proxyKey && options.proxy[proxyKey]) {
        return (0, _proxyHandle.proxyHandle)(req, res, options, proxyKey);
      }
    } else {
      if (proxyKey && options.proxy[proxyKey]) {
        return (0, _proxyHandle.proxyHandle)(req, res, options, proxyKey);
      } else if (mocker[mockerKey]) {
        return (0, _mockerHandle.mockerHandle)({
          req: req,
          res: res,
          next: next,
          mocker: mocker,
          options: options,
          mockerKey: mockerKey
        });
      }
    }

    next();
  });
  /**
   * The old module's resources to be released.
   * @param modulePath 
   */

  function cleanCache(modulePath) {
    // The entry file does not have a .js suffix,
    // causing the module's resources not to be released.
    // https://github.com/jaywcjlove/webpack-api-mocker/issues/30
    try {
      modulePath = require.resolve(modulePath);
    } catch (e) {}

    var module = require.cache[modulePath];
    if (!module) return; // remove reference in module.parent

    if (module.parent) {
      module.parent.children.splice(module.parent.children.indexOf(module), 1);
    } // https://github.com/jaywcjlove/mocker-api/issues/42


    (0, _clearModule["default"])(modulePath);
  }
  /**
   * Merge multiple Mockers
   */


  function getConfig() {
    return watchFiles.reduce(function (mocker, file) {
      var mockerItem = require(file);

      return Object.assign(mocker, mockerItem["default"] ? mockerItem["default"] : mockerItem);
    }, {});
  }

  return function (req, res, next) {
    next();
  };
}

module.exports = exports.default;
//# sourceMappingURL=index.js.map